//+------------------------------------------------------------------+
//|                                                  Custom MACD.mq4 |
//|                   Copyright 2005-2014, MetaQuotes Software Corp. |
//|                                              http://www.mql4.com |
//+------------------------------------------------------------------+
#property copyright   "2005-2014, MetaQuotes Software Corp."
#property link        "http://www.mql4.com"
#property description "Moving Averages Convergence/Divergence"
#property strict
//---
#include <MovingAverages.mqh>
//--- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 2
#property  indicator_color1  Silver
#property  indicator_color2  Red
#property  indicator_width1  2
//--- indicator parameters
extern int InpFastEMA=12;   // Fast EMA Period
extern int InpSlowEMA=26;   // Slow EMA Period
extern int InpSignalSMA=9;  // Signal SMA Period
//--- indicator buffers
double    ExtMacdBuffer[];
double    ExtSignalBuffer[];
//--- right input parameters flag
bool      ExtParameters=false;
int prevCalculated=0;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit(void)
  {
   IndicatorDigits(Digits+1);
//--- drawing settings
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexDrawBegin(1,InpSignalSMA);
//--- indicator buffers mapping
   SetIndexBuffer(0,ExtMacdBuffer);
   SetIndexBuffer(1,ExtSignalBuffer);
//--- name for DataWindow and indicator subwindow label
   IndicatorShortName("MACD("+IntegerToString(InpFastEMA)+","+IntegerToString(InpSlowEMA)+","+IntegerToString(InpSignalSMA)+")");
   SetIndexLabel(0,"MACD");
   SetIndexLabel(1,"Signal");
//--- check for input parameters
   if(InpFastEMA<=1 || InpSlowEMA<=1 || InpSignalSMA<=1 || InpFastEMA>=InpSlowEMA)
     {
      Print("Wrong input parameters");
      ExtParameters=false;
      return(INIT_FAILED);
     }
   else
      ExtParameters=true;
//---
   if(ObjectFind(ChartID(),"Values")==-1)
      if(!ObjectCreate(ChartID(),"Values",OBJ_TEXT,0,0,0))
         return(INIT_FAILED);
   prevCalculated=0;
//--- initialization done
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int OnCalculate (const int rates_total,
                 const int prev_calculated,
                 const datetime& time[],
                 const double& open[],
                 const double& high[],
                 const double& low[],
                 const double& close[],
                 const long& tick_volume[],
                 const long& volume[],
                 const int& spread[])
  {
   int i,limit;
//---
   if(rates_total<=InpSignalSMA || !ExtParameters)
      return(0);
//--- last counted bar will be recounted
   limit=rates_total-prevCalculated;
   if(prevCalculated>0)
      limit++;
//--- macd counted in the 1-st buffer
   for(i=0; i<limit; i++)
      ExtMacdBuffer[i]=iMA(NULL,0,InpFastEMA,0,MODE_EMA,PRICE_CLOSE,i)-
                       iMA(NULL,0,InpSlowEMA,0,MODE_EMA,PRICE_CLOSE,i);
//--- signal line counted in the 2-nd buffer
   SimpleMAOnBuffer(rates_total,prevCalculated,0,InpSignalSMA,ExtMacdBuffer,ExtSignalBuffer);
//---
   if(rates_total!=prevCalculated)
     {
      ObjectSetInteger(ChartID(),"Values",OBJPROP_TIME,time[0]);
      ObjectSetDouble(ChartID(),"Values",OBJPROP_PRICE,ChartGetDouble(ChartID(),CHART_PRICE_MIN)+100*_Point);
      ObjectSetString(ChartID(),"Values",OBJPROP_TEXT,StringFormat("%d, %d, %d",InpFastEMA,InpSlowEMA,InpSignalSMA));
      ObjectSetInteger(ChartID(),"Values",OBJPROP_COLOR,clrWhite);
     }
//--- done
   return(prevCalculated=rates_total);
  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
//---
   static bool clicked=false;
   if(id==CHARTEVENT_OBJECT_CLICK)
     {
      if(clicked)
        { 
         string values=ObjectGetString(ChartID(),"Values",OBJPROP_TEXT);
         int a=0,b=0;
         for(int i=0;i<StringLen(values);i++)
           {
            a=StringFind(values,",");
            b=StringFind(values,",");
           }
         InpFastEMA=(int)StringSubstr(values,0,a);
         InpSlowEMA=(int)StringSubstr(values,a+1,b-a);
         ObjectSetInteger(ChartID(),"Values",OBJPROP_SELECTED,false);
         OnInit();
         if(!prevCalculated)
           {
            long volume[1];
            int spread[1];
            OnCalculate(Bars,0,Time,Open,High,Low,Close,Volume,volume,spread);
           }
         clicked=false;
        }
      else
        { 
         ObjectSetInteger(ChartID(),"Values",OBJPROP_SELECTED,true);
         clicked=true;
        }
     } 
  }
//+------------------------------------------------------------------+